#!/usr/bin/env python3
"""Writes bounding_box_and_polygon.png that illustrates
"""
from PIL import Image, ImageDraw

from pyzbar.pyzbar import decode


image = Image.open('pyzbar/tests/qrcode_rotated.png').convert('RGB')
draw = ImageDraw.Draw(image)
for barcode in decode(image):
    rect = barcode.rect
    draw.rectangle(
        (
            (rect.left, rect.top),
            (rect.left + rect.width, rect.top + rect.height)
        ),
        outline='#0080ff'
    )

    draw.polygon(barcode.polygon, outline='#e945ff')


image.save('bounding_box_and_polygon.png')



import unittest

from pyzbar.locations import bounding_box, convex_hull, Rect


class TestLocations(unittest.TestCase):
    def test_bounding_box(self):
        self.assertRaises(ValueError, bounding_box, [])
        self.assertEqual(
            Rect(left=0, top=0, width=0, height=0),
            bounding_box([(0, 0)])
        )
        self.assertEqual(
            Rect(left=37, top=550, width=324, height=76),
            bounding_box([(37, 551), (37, 625), (361, 626), (361, 550)])
        )

    def test_convex_hull_empty(self):
        self.assertEqual([], convex_hull([]))

    def test_convex_square(self):
        points = [(0, 0), (0, 1), (1, 1), (1, 0)]
        self.assertEqual(points, convex_hull(points)),

    def test_convex_duplicates(self):
        points = [(0, 0), (0, 1), (1, 1), (1, 0)]
        self.assertEqual(points, convex_hull(points * 10)),

    def test_other(self):
        # Taken from
        # https://codegolf.stackexchange.com/questions/11035/find-the-convex-hull-of-a-set-of-2d-points
        res = convex_hull([(1, 1), (2, 2), (3, 3), (1, 3)])
        self.assertEqual([(1, 1), (1, 3), (3, 3)], res)

        res = convex_hull([
            (4.4, 14), (6.7, 15.25), (6.9, 12.8), (2.1, 11.1), (9.5, 14.9),
            (13.2, 11.9), (10.3, 12.3), (6.8, 9.5), (3.3, 7.7), (0.6, 5.1),
            (5.3, 2.4), (8.45, 4.7), (11.5, 9.6), (13.8, 7.3), (12.9, 3.1),
            (11, 1.1)
        ])

        expected = [
            (0.6, 5.1), (2.1, 11.1), (4.4, 14), (6.7, 15.25), (9.5, 14.9),
            (13.2, 11.9), (13.8, 7.3), (12.9, 3.1), (11, 1.1), (5.3, 2.4)
        ]
        self.assertEqual(expected, res)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
from __future__ import print_function

import argparse
import sys

import pyzbar
from pyzbar.pyzbar import decode


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    parser = argparse.ArgumentParser(
        description='Reads barcodes in images, using the zbar library'
    )
    parser.add_argument('image', nargs='+')
    parser.add_argument(
        '-v', '--version', action='version',
        version='%(prog)s ' + pyzbar.__version__
    )
    args = parser.parse_args(args)

    from PIL import Image

    for image in args.image:
        for barcode in decode(Image.open(image)):
            print(barcode.data)


if __name__ == '__main__':
    main()



import sys
import unittest

from pathlib import Path
from contextlib import contextmanager

# TODO Would io.StringIO not work in all cases?
try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

from pyzbar.scripts.read_zbar import main


@contextmanager
def capture_stdout():
    sys.stdout, old_stdout = StringIO(), sys.stdout
    try:
        yield sys.stdout
    finally:
        sys.stdout = old_stdout


class TestReadZbar(unittest.TestCase):
    def test_read_qrcode(self):
        "Read QR code"
        with capture_stdout() as stdout:
            main([str(Path(__file__).parent.joinpath('qrcode.png'))])

        if 2 == sys.version_info[0]:
            expected = "Thalassiodracon"
        else:
            expected = "b'Thalassiodracon'"

        self.assertEqual(expected, stdout.getvalue().strip())

    def test_read_code128(self):
        "Read CODE 128 barcodes"
        with capture_stdout() as stdout:
            main([str(Path(__file__).parent.joinpath('code128.png'))])

        if 2 == sys.version_info[0]:
            expected = "Foramenifera\nRana temporaria"
        else:
            expected = "b'Foramenifera'\nb'Rana temporaria'"

        self.assertEqual(expected, stdout.getvalue().strip())


if __name__ == '__main__':
    unittest.main()



"""Low-level wrapper around zbar's interface
"""
from ctypes import (
    c_ubyte, c_char_p, c_int, c_uint, c_ulong, c_void_p, Structure,
    CFUNCTYPE, POINTER
)
from enum import IntEnum, unique

from . import zbar_library


__all__ = [
    'EXTERNAL_DEPENDENCIES', 'LIBZBAR', 'ZBarConfig', 'ZBarSymbol',
    'zbar_image_create', 'zbar_image_destroy', 'zbar_image_first_symbol',
    'zbar_image_scanner_create', 'zbar_image_scanner_destroy',
    'zbar_image_scanner_set_config', 'zbar_image_set_data',
    'zbar_image_set_format', 'zbar_image_set_size', 'zbar_scan_image',
    'zbar_symbol_get_data', 'zbar_symbol_get_loc_size',
    'zbar_symbol_get_loc_x', 'zbar_symbol_get_loc_y', 'zbar_symbol_next'
]

# Globals populated in load_libzbar
LIBZBAR = None
"""ctypes.CDLL
"""

EXTERNAL_DEPENDENCIES = []
"""List of instances of ctypes.CDLL. Helpful when freezing.
"""

# Types
c_ubyte_p = POINTER(c_ubyte)
c_uint_p = POINTER(c_uint)
c_ulong_p = POINTER(c_ulong)
"""unsigned char* type
"""


# Defines and enums
@unique
class ZBarSymbol(IntEnum):
    NONE = 0          # /**< no symbol decoded */
    PARTIAL = 1       # /**< intermediate status */
    EAN2 = 2          # /**< GS1 2-digit add-on */
    EAN5 = 5          # /**< GS1 5-digit add-on */
    EAN8 = 8          # /**< EAN-8 */
    UPCE = 9          # /**< UPC-E */
    ISBN10 = 10       # /**< ISBN-10 (from EAN-13). @since 0.4 */
    UPCA = 12         # /**< UPC-A */
    EAN13 = 13        # /**< EAN-13 */
    ISBN13 = 14       # /**< ISBN-13 (from EAN-13). @since 0.4 */
    COMPOSITE = 15    # /**< EAN/UPC composite */
    I25 = 25          # /**< Interleaved 2 of 5. @since 0.4 */
    DATABAR = 34      # /**< GS1 DataBar (RSS). @since 0.11 */
    DATABAR_EXP = 35  # /**< GS1 DataBar Expanded. @since 0.11 */
    CODABAR = 38      # /**< Codabar. @since 0.11 */
    CODE39 = 39       # /**< Code 39. @since 0.4 */
    PDF417 = 57       # /**< PDF417. @since 0.6 */
    QRCODE = 64       # /**< QR Code. @since 0.10 */
    CODE93 = 93       # /**< Code 93. @since 0.11 */
    CODE128 = 128     # /**< Code 128 */


@unique
class ZBarConfig(IntEnum):
    CFG_ENABLE = 0          # /**< enable symbology/feature */
    CFG_ADD_CHECK = 1       # /**< enable check digit when optional */
    CFG_EMIT_CHECK = 2      # /**< return check digit when present */
    CFG_ASCII = 3           # /**< enable full ASCII character set */
    CFG_NUM = 4             # /**< number of boolean decoder configs */

    CFG_MIN_LEN = 0x20      # /**< minimum data length for valid decode */
    CFG_MAX_LEN = 0x21      # /**< maximum data length for valid decode */

    CFG_UNCERTAINTY = 0x40  # /**< required video consistency frames */

    CFG_POSITION = 0x80     # /**< enable scanner to collect position data */

    CFG_X_DENSITY = 0x100   # /**< image scanner vertical scan density */
    CFG_Y_DENSITY = 0x101   # /**< image scanner horizontal scan density */


# Structs
class zbar_image_scanner(Structure):
    """Opaque C++ class with private implementation
    """
    pass


class zbar_image(Structure):
    """Opaque C++ class with private implementation
    """
    pass


class zbar_symbol(Structure):
    """Opaque C++ class with private implementation

    The first item in the structure is an integeger value in the ZBarSymbol
    enumeration.
    """
    _fields_ = [
        ('type', c_int),
    ]


def load_libzbar():
    """Loads the zbar shared library and its dependencies.

    Populates the globals LIBZBAR and EXTERNAL_DEPENDENCIES.
    """
    global LIBZBAR
    global EXTERNAL_DEPENDENCIES
    if not LIBZBAR:
        libzbar, dependencies = zbar_library.load()
        LIBZBAR = libzbar
        EXTERNAL_DEPENDENCIES = [LIBZBAR] + dependencies

    return LIBZBAR


# Function signatures
def zbar_function(fname, restype, *args):
    """Returns a foreign function exported by `zbar`.

    Args:
        fname (:obj:`str`): Name of the exported function as string.
        restype (:obj:): Return type - one of the `ctypes` primitive C data
        types.
        *args: Arguments - a sequence of `ctypes` primitive C data types.

    Returns:
        cddl.CFunctionType: A wrapper around the function.
    """
    prototype = CFUNCTYPE(restype, *args)
    return prototype((fname, load_libzbar()))


zbar_version = zbar_function(
    'zbar_version',
    c_int,
    c_uint_p,    # major,
    c_uint_p,    # minor
)

zbar_set_verbosity = zbar_function(
    'zbar_set_verbosity',
    None,
    c_int
)

zbar_image_scanner_create = zbar_function(
    'zbar_image_scanner_create',
    POINTER(zbar_image_scanner)
)

zbar_image_scanner_destroy = zbar_function(
    'zbar_image_scanner_destroy',
    None,
    POINTER(zbar_image_scanner)
)

zbar_parse_config = zbar_function(
    'zbar_parse_config',
    c_int,
    c_char_p,          # config_string,
    POINTER(c_int),    # symbology - values in ZBarSymbol
    POINTER(c_int),    # config - values in ZBarConfig
    POINTER(c_int),    # value
)

zbar_image_scanner_set_config = zbar_function(
    'zbar_image_scanner_set_config',
    c_int,
    POINTER(zbar_image_scanner),  # scanner
    c_int,                        # symbology - values in ZBarSymbol
    c_int,                        # config - values in ZBarConfig
    c_int                         # value
)

zbar_image_create = zbar_function(
    'zbar_image_create',
    POINTER(zbar_image)
)

zbar_image_destroy = zbar_function(
    'zbar_image_destroy',
    None,
    POINTER(zbar_image)
)

zbar_image_set_format = zbar_function(
    'zbar_image_set_format',
    None,
    POINTER(zbar_image),
    c_uint
)

zbar_image_set_size = zbar_function(
    'zbar_image_set_size',
    None,
    POINTER(zbar_image),
    c_uint,     # width
    c_uint      # height
)

zbar_image_set_data = zbar_function(
    'zbar_image_set_data',
    None,
    POINTER(zbar_image),
    c_void_p,   # data
    c_ulong,    # raw_image_data_length
    c_void_p    # A function pointer(!)
)

zbar_scan_image = zbar_function(
    'zbar_scan_image',
    c_int,
    POINTER(zbar_image_scanner),
    POINTER(zbar_image)
)

zbar_image_first_symbol = zbar_function(
    'zbar_image_first_symbol',
    POINTER(zbar_symbol),
    POINTER(zbar_image)
)

zbar_symbol_get_data_length = zbar_function(
    'zbar_symbol_get_data_length',
    c_uint,
    POINTER(zbar_symbol)
)

zbar_symbol_get_data = zbar_function(
    'zbar_symbol_get_data',
    c_char_p,
    POINTER(zbar_symbol)
)

zbar_symbol_get_loc_size = zbar_function(
    'zbar_symbol_get_loc_size',
    c_uint,
    POINTER(zbar_symbol)
)

zbar_symbol_get_loc_x = zbar_function(
    'zbar_symbol_get_loc_x',
    c_int,
    POINTER(zbar_symbol),
    c_uint
)

zbar_symbol_get_loc_y = zbar_function(
    'zbar_symbol_get_loc_y',
    c_int,
    POINTER(zbar_symbol),
    c_uint
)

zbar_symbol_next = zbar_function(
    'zbar_symbol_next',
    POINTER(zbar_symbol),
    POINTER(zbar_symbol)
)



"""Loads zbar and its dependencies.
"""
import platform
import sys

from ctypes import cdll
from ctypes.util import find_library
from pathlib import Path

__all__ = ['load']


def _windows_fnames():
    """For convenience during development and to aid debugging, the DLL names
    are specific to the bit depth of interpreter.

    This logic has its own function to make testing easier
    """
    # 'libzbar-64.dll' and 'libzbar-32.dll' each have a dependent DLL -
    # 'libiconv.dll' and 'libiconv-2.dll' respectively.
    if sys.maxsize > 2**32:
        # 64-bit
        fname = 'libzbar-64.dll'
        dependencies = ['libiconv.dll']
    else:
        # 32-bit
        fname = 'libzbar-32.dll'
        dependencies = ['libiconv-2.dll']

    return fname, dependencies


def load():
    """Loads the libzar shared library and its dependencies.
    """
    if 'Windows' == platform.system():
        # Possible scenarios here
        #   1. Run from source, DLLs are in pyzbar directory
        #       cdll.LoadLibrary() imports DLLs in repo root directory
        #   2. Wheel install into CPython installation
        #       cdll.LoadLibrary() imports DLLs in package directory
        #   3. Wheel install into virtualenv
        #       cdll.LoadLibrary() imports DLLs in package directory
        #   4. Frozen
        #       cdll.LoadLibrary() imports DLLs alongside executable
        fname, dependencies = _windows_fnames()

        def load_objects(directory):
            # Load dependencies before loading libzbar dll
            deps = [
                cdll.LoadLibrary(str(directory.joinpath(dep)))
                for dep in dependencies
            ]
            libzbar = cdll.LoadLibrary(str(directory.joinpath(fname)))
            return deps, libzbar

        try:
            dependencies, libzbar = load_objects(Path(''))
        except OSError:
            dependencies, libzbar = load_objects(Path(__file__).parent)
    else:
        # Assume a shared library on the path
        path = find_library('zbar')
        if not path:
            raise ImportError('Unable to find zbar shared library')
        libzbar = cdll.LoadLibrary(path)
        dependencies = []

    return libzbar, dependencies



import unittest

from pathlib import Path

try:
    from unittest.mock import call, patch
except ImportError:
    # Python 2
    from mock import call, patch

from pyzbar import zbar_library


class TestLoad(unittest.TestCase):
    def setUp(self):
        self.addCleanup(patch.stopall)
        self.cdll = patch(
            'pyzbar.zbar_library.cdll', autospec=True
        ).start()
        self.find_library = patch(
            'pyzbar.zbar_library.find_library', autospec=True
        ).start()
        self.platform = patch(
            'pyzbar.zbar_library.platform', autospec=True
        ).start()
        self.windows_fnames = patch(
            'pyzbar.zbar_library._windows_fnames', autospec=True,
            return_value=('dll fname', ['dependency fname'])
        ).start()

    def test_found_non_windows(self):
        "zbar loaded ok on non-Windows platform"
        self.platform.system.return_value = 'Not windows'

        res = zbar_library.load()

        self.platform.system.assert_called_once_with()
        self.find_library.assert_called_once_with('zbar')
        self.cdll.LoadLibrary.assert_called_once_with(
            self.find_library.return_value
        )

        self.assertEqual((self.cdll.LoadLibrary.return_value, []), res)
        self.assertEqual(0, self.windows_fnames.call_count)

    def test_not_found_non_windows(self):
        "zbar not found on non-Windows platform"
        self.platform.system.return_value = 'Not windows'
        self.find_library.return_value = None

        self.assertRaises(ImportError, zbar_library.load)

        self.platform.system.assert_called_once_with()
        self.find_library.assert_called_once_with('zbar')

    def test_found_windows(self):
        "zbar found on Windows"
        self.platform.system.return_value = 'Windows'

        res = zbar_library.load()

        self.platform.system.assert_called_once_with()
        self.cdll.LoadLibrary.assert_has_calls([
            call('dependency fname'),
            call('dll fname'),
        ])
        self.assertEqual(
            (
                self.cdll.LoadLibrary.return_value,
                [self.cdll.LoadLibrary.return_value]
            ),
            res
        )

    def test_found_second_attempt_windows(self):
        "zbar found on the second attempt on Windows"
        self.platform.system.return_value = 'Windows'
        self.cdll.LoadLibrary.side_effect = [
            OSError,                # First call does not load dependent DLL
            'loaded dependency',    # Second call loads dependent DLL
            'loaded zbar',          # Third call loads dependent DLL
        ]

        res = zbar_library.load()

        self.platform.system.assert_called_once_with()
        self.cdll.LoadLibrary.assert_has_calls([
            call('dependency fname'),
            call(str(Path(zbar_library.__file__).parent.joinpath(
                'dependency fname'
            ))),
            call(str(Path(zbar_library.__file__).parent.joinpath(
                'dll fname'
            ))),
        ])

        self.assertEqual(('loaded zbar', ['loaded dependency']), res)

    def test_not_found_windows(self):
        "zbar not found on Windows"
        self.platform.system.return_value = 'Windows'
        self.cdll.LoadLibrary.side_effect = OSError

        self.assertRaises(OSError, zbar_library.load)

        self.platform.system.assert_called_once_with()
        # Two attempts at loading
        self.cdll.LoadLibrary.assert_has_calls([
            call('dependency fname'),
            call(str(Path(zbar_library.__file__).parent.joinpath(
                'dependency fname'
            ))),
        ])


class TestWindowsFnames(unittest.TestCase):
    def setUp(self):
        self.addCleanup(patch.stopall)
        self.sys = patch('pyzbar.zbar_library.sys', autospec=True).start()

    def test_32bit(self):
        self.sys.maxsize = 2**32
        self.assertEqual(
            ('libzbar-32.dll', ['libiconv-2.dll']),
            zbar_library._windows_fnames()
        )

    def test_64bit(self):
        # This is a 'long' on a 32-bit interpreter
        self.sys.maxsize = 2**32 + 1
        self.assertEqual(
            ('libzbar-64.dll', ['libiconv.dll']),
            zbar_library._windows_fnames()
        )


if __name__ == '__main__':
    unittest.main()
